@@ -1,4 +1,5 @@
 require File.dirname(__FILE__) + '/helper'
+require 'date'
 
 class HelpersTest < Test::Unit::TestCase
   def test_default
@@ -57,7 +58,7 @@ class HelpersTest < Test::Unit::TestCase
       get '/'
       assert_equal 302, status
       assert_equal '', body
-      assert_equal '/foo', response['Location']
+      assert_equal 'http://example.org/foo', response['Location']
     end
 
     it 'uses the code given when specified' do
@@ -71,7 +72,7 @@ class HelpersTest < Test::Unit::TestCase
       get '/'
       assert_equal 301, status
       assert_equal '', body
-      assert_equal '/foo', response['Location']
+      assert_equal 'http://example.org/foo', response['Location']
     end
 
     it 'redirects back to request.referer when passed back' do
@@ -84,7 +85,31 @@ class HelpersTest < Test::Unit::TestCase
       request = Rack::MockRequest.new(@app)
       response = request.get('/try_redirect', 'HTTP_REFERER' => '/foo')
       assert_equal 302, response.status
-      assert_equal '/foo', response['Location']
+      assert_equal 'http://example.org/foo', response['Location']
+    end
+
+    it 'redirects using a non-standard HTTP port' do
+      mock_app {
+        get '/' do
+          redirect '/foo'
+        end
+      }
+
+      request = Rack::MockRequest.new(@app)
+      response = request.get('/', 'SERVER_PORT' => '81')
+      assert_equal 'http://example.org:81/foo', response['Location']
+    end
+
+    it 'redirects using a non-standard HTTPS port' do
+      mock_app {
+        get '/' do
+          redirect '/foo'
+        end
+      }
+
+      request = Rack::MockRequest.new(@app)
+      response = request.get('/', 'SERVER_PORT' => '444')
+      assert_equal 'http://example.org:444/foo', response['Location']
     end
   end
 
@@ -142,6 +167,19 @@ class HelpersTest < Test::Unit::TestCase
       assert_equal 404, status
       assert_equal '', body
     end
+
+    it 'does not set a X-Cascade header' do
+      mock_app {
+        get '/' do
+          not_found
+          fail 'not_found should halt'
+        end
+      }
+
+      get '/'
+      assert_equal 404, status
+      assert_equal nil, response.headers['X-Cascade']
+    end
   end
 
   describe 'headers' do
@@ -188,41 +226,61 @@ class HelpersTest < Test::Unit::TestCase
 
     it 'creates a new session when none provided' do
       mock_app {
+        enable :sessions
+
         get '/' do
           assert session.empty?
           session[:foo] = 'bar'
-          'Hi'
+          redirect '/hi'
+        end
+
+        get '/hi' do
+          "hi #{session[:foo]}"
         end
       }
 
       get '/'
-      assert_equal 'Hi', body
+      follow_redirect!
+      assert_equal 'hi bar', body
     end
   end
 
-  describe 'media_type' do
+  describe 'mime_type' do
     include Sinatra::Helpers
 
-    it "looks up media types in Rack's MIME registry" do
+    it "looks up mime types in Rack's MIME registry" do
       Rack::Mime::MIME_TYPES['.foo'] = 'application/foo'
-      assert_equal 'application/foo', media_type('foo')
-      assert_equal 'application/foo', media_type('.foo')
-      assert_equal 'application/foo', media_type(:foo)
+      assert_equal 'application/foo', mime_type('foo')
+      assert_equal 'application/foo', mime_type('.foo')
+      assert_equal 'application/foo', mime_type(:foo)
     end
 
     it 'returns nil when given nil' do
-      assert media_type(nil).nil?
+      assert mime_type(nil).nil?
     end
 
     it 'returns nil when media type not registered' do
-      assert media_type(:bizzle).nil?
+      assert mime_type(:bizzle).nil?
     end
 
     it 'returns the argument when given a media type string' do
-      assert_equal 'text/plain', media_type('text/plain')
+      assert_equal 'text/plain', mime_type('text/plain')
     end
   end
 
+  test 'Base.mime_type registers mime type' do
+    mock_app {
+      mime_type :foo, 'application/foo'
+
+      get '/' do
+        "foo is #{mime_type(:foo)}"
+      end
+    }
+
+    get '/'
+    assert_equal 'foo is application/foo', body
+  end
+
   describe 'content_type' do
     it 'sets the Content-Type header' do
       mock_app {
@@ -233,21 +291,21 @@ class HelpersTest < Test::Unit::TestCase
       }
 
       get '/'
-      assert_equal 'text/plain', response['Content-Type']
+      assert_equal 'text/plain;charset=utf-8', response['Content-Type']
       assert_equal 'Hello World', body
     end
 
     it 'takes media type parameters (like charset=)' do
       mock_app {
         get '/' do
-          content_type 'text/html', :charset => 'utf-8'
+          content_type 'text/html', :charset => 'latin1'
           "<h1>Hello, World</h1>"
         end
       }
 
       get '/'
       assert ok?
-      assert_equal 'text/html;charset=utf-8', response['Content-Type']
+      assert_equal 'text/html;charset=latin1', response['Content-Type']
       assert_equal "<h1>Hello, World</h1>", body
     end
 
@@ -262,7 +320,7 @@ class HelpersTest < Test::Unit::TestCase
 
       get '/foo.xml'
       assert ok?
-      assert_equal 'application/foo', response['Content-Type']
+      assert_equal 'application/foo;charset=utf-8', response['Content-Type']
       assert_equal 'I AM FOO', body
     end
 
@@ -308,7 +366,19 @@ class HelpersTest < Test::Unit::TestCase
     it 'sets the Content-Type response header if a mime-type can be located' do
       send_file_app
       get '/file.txt'
-      assert_equal 'text/plain', response['Content-Type']
+      assert_equal 'text/plain;charset=utf-8', response['Content-Type']
+    end
+
+    it 'sets the Content-Type response header if type option is set to a file extesion' do
+      send_file_app :type => 'html'
+      get '/file.txt'
+      assert_equal 'text/html;charset=utf-8', response['Content-Type']
+    end
+
+    it 'sets the Content-Type response header if type option is set to a mime type' do
+      send_file_app :type => 'application/octet-stream'
+      get '/file.txt'
+      assert_equal 'application/octet-stream;charset=utf-8', response['Content-Type']
     end
 
     it 'sets the Content-Length response header' do
@@ -352,34 +422,147 @@ class HelpersTest < Test::Unit::TestCase
     end
   end
 
-  describe 'last_modified' do
+  describe 'cache_control' do
     setup do
-      now = Time.now
       mock_app {
         get '/' do
-          body { 'Hello World' }
-          last_modified now
-          'Boo!'
+          cache_control :public, :no_cache, :max_age => 60
+          'Hello World'
         end
       }
-      @now = now
     end
 
-    it 'sets the Last-Modified header to a valid RFC 2616 date value' do
+    it 'sets the Cache-Control header' do
       get '/'
-      assert_equal @now.httpdate, response['Last-Modified']
+      assert_equal ['public', 'no-cache', 'max-age=60'], response['Cache-Control'].split(', ')
+    end
+  end
+
+  describe 'expires' do
+    setup do
+      mock_app {
+        get '/' do
+          expires 60, :public, :no_cache
+          'Hello World'
+        end
+      }
     end
 
-    it 'returns a body when conditional get misses' do
+    it 'sets the Cache-Control header' do
       get '/'
-      assert_equal 200, status
-      assert_equal 'Boo!', body
+      assert_equal ['public', 'no-cache', 'max-age=60'], response['Cache-Control'].split(', ')
     end
 
-    it 'halts when a conditional GET matches' do
-      get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => @now.httpdate }
-      assert_equal 304, status
-      assert_equal '', body
+    it 'sets the Expires header' do
+      get '/'
+      assert_not_nil response['Expires']
+    end
+  end
+
+  describe 'last_modified' do
+    it 'ignores nil' do
+      mock_app do
+        get '/' do last_modified nil; 200; end
+      end
+
+      get '/'
+      assert ! response['Last-Modified']
+    end
+
+    [Time, DateTime].each do |klass|
+      describe "with #{klass.name}" do
+        setup do
+          last_modified_time = klass.now
+          mock_app do
+            get '/' do
+              last_modified last_modified_time
+              'Boo!'
+            end
+          end
+          @last_modified_time = Time.parse last_modified_time.to_s
+        end
+
+        # fixes strange missing test error when running complete test suite.
+        it("does not complain about missing tests") { }
+
+        context "when there's no If-Modified-Since header" do
+          it 'sets the Last-Modified header to a valid RFC 2616 date value' do
+            get '/'
+            assert_equal @last_modified_time.httpdate, response['Last-Modified']
+          end
+
+          it 'conditional GET misses and returns a body' do
+            get '/'
+            assert_equal 200, status
+            assert_equal 'Boo!', body
+          end
+        end
+
+        context "when there's an invalid If-Modified-Since header" do
+          it 'sets the Last-Modified header to a valid RFC 2616 date value' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => 'a really weird date' }
+            assert_equal @last_modified_time.httpdate, response['Last-Modified']
+          end
+
+          it 'conditional GET misses and returns a body' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => 'a really weird date' }
+            assert_equal 200, status
+            assert_equal 'Boo!', body
+          end
+        end
+
+        context "when the resource has been modified since the If-Modified-Since header date" do
+          it 'sets the Last-Modified header to a valid RFC 2616 date value' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => (@last_modified_time - 1).httpdate }
+            assert_equal @last_modified_time.httpdate, response['Last-Modified']
+          end
+
+          it 'conditional GET misses and returns a body' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => (@last_modified_time - 1).httpdate }
+            assert_equal 200, status
+            assert_equal 'Boo!', body
+          end
+
+          it 'does not rely on string comparison' do
+            mock_app do
+              get '/compare' do
+                last_modified "Mon, 18 Oct 2010 20:57:11 GMT"
+                "foo"
+              end
+            end
+
+            get '/compare', {}, { 'HTTP_IF_MODIFIED_SINCE' => 'Sun, 26 Sep 2010 23:43:52 GMT' }
+            assert_equal 200, status
+            assert_equal 'foo', body
+          end
+        end
+
+        context "when the resource has been modified on the exact If-Modified-Since header date" do
+          it 'sets the Last-Modified header to a valid RFC 2616 date value' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => @last_modified_time.httpdate }
+            assert_equal @last_modified_time.httpdate, response['Last-Modified']
+          end
+
+          it 'conditional GET matches and halts' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => @last_modified_time.httpdate }
+            assert_equal 304, status
+            assert_equal '', body
+          end
+        end
+
+        context "when the resource hasn't been modified since the If-Modified-Since header date" do
+          it 'sets the Last-Modified header to a valid RFC 2616 date value' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => (@last_modified_time + 1).httpdate }
+            assert_equal @last_modified_time.httpdate, response['Last-Modified']
+          end
+
+          it 'conditional GET matches and halts' do
+            get '/', {}, { 'HTTP_IF_MODIFIED_SINCE' => (@last_modified_time + 1).httpdate }
+            assert_equal 304, status
+            assert_equal '', body
+          end
+        end
+      end
     end
   end
 
