@@ -1,6 +1,6 @@
 require File.dirname(__FILE__) + '/helper'
 
-describe 'Sinatra' do
+class SinatraTest < Test::Unit::TestCase
   it 'creates a new Sinatra::Base subclass on new' do
     app =
       Sinatra.new do
@@ -10,4 +10,8 @@ describe 'Sinatra' do
       end
     assert_same Sinatra::Base, app.superclass
   end
+  
+  it "responds to #template_cache" do
+    assert_kind_of Tilt::Cache, Sinatra::Base.new.template_cache
+  end
 end
